<?php
namespace EMS\PortalFuncionario\Geral\Services;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Util\Request;
use EMS\PortalFuncionario\Util\Messages;
use EMS\PortalFuncionario\Util\Util;
use EMS\PortalFuncionario\Geral\Repository\OperadorRepository;
use EMS\PortalFuncionario\Geral\Repository\ParamOrgRepository;

class OperadorService extends Crud {
    /** @var OperadorRepository */
    protected $operadorRepository;

    /** @var ParamOrgRepository */
    protected $paramOrgRepository;

    public const OPERATOR_DEFAULT_HASH = 'eea79e8ff3dd8b6e7160c5cc509c9bb6';
    public const DEFAULT_PASSWORD = 'HJ6bKz1mznUruX26R3Wp';

    public function __construct(EntityManager $entityManager, OperadorRepository $operadorRepository, ParamOrgRepository $paramOrgRepository) {
        $this->entityManager        = $entityManager;
        $this->operadorRepository   = $operadorRepository;
        $this->paramOrgRepository   = $paramOrgRepository;
    }

    public function get(
        int $nrOrg,
        string $cdOperador
    ) {
        $result = $this->operadorRepository->get($nrOrg, $cdOperador);
        return $result;
    }

    public function getDadosOperador(
        string $cdoperador,
        string $dtNascPessoa        = null,
        string $dtAdmissaoVinc      = null,
        string $cpfPessoa           = null,
        string $idConfirmrecsenport = null
    ) {
        try {
            $dadosOperador = $this->operadorRepository->getDadosOperador(
                $cdoperador,
                $dtNascPessoa,
                $dtAdmissaoVinc,
                $cpfPessoa,
                $idConfirmrecsenport
            );
            if (empty($dadosOperador) || empty($dadosOperador[0])) {
                throw new \Exception('Operador não encontrado');
            }
            return $dadosOperador[0];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLogin(string $nrvinculom, string $nrorg) {
        try {
            $loginValido = !empty($this->operadorRepository->validaLogin(
                $nrvinculom,
                $nrorg
            ));
            return $loginValido;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaLoginOperador(
        string $cdoperador,
        string $dtNascPessoa        = null,
        string $dtAdmissaoVinc      = null,
        string $cpfPessoa           = null,
        string $idConfirmrecsenport = null
    ) {
        try {
            $dadosOperador = $this->getDadosOperador(
                $cdoperador,
                $dtNascPessoa,
                $dtAdmissaoVinc,
                $cpfPessoa,
                $idConfirmrecsenport
            );

            $loginValido = !empty($dadosOperador);
            $dadosOperador['LOGINVALIDO'] = $loginValido;

            $nrorg = $dadosOperador['NRORG'];
            $controleAcessoAtivo = $this->controleAcessoAtivo($nrorg);
    
            if($controleAcessoAtivo) {
                $permissoes = $this->getPermissoesModuloOperador($nrorg, $cdoperador);
    
                $dadosOperador['PERMISSOES'] = $permissoes;
            }

            return $dadosOperador;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getBeforeForgotPasswordInfo(
        string  $email
    ): Array {
        $params = [
            'P_DSEMAILOPER' => $email,
        ];

        $dadosOperador = $this->operadorRepository->getBeforeForgotPasswordInfo($params);

        return $dadosOperador;
    }

    public function clearOperatorPassword(
        string  $nrOrg,
        string  $cdOperador
    ): bool {
        $params = [
            'P_NRORG'       => $nrOrg,
            'P_CDOPERADOR'  => $cdOperador,
        ];

        $dadosOperador = $this->operadorRepository->clearOperatorPassword($params);

        return $dadosOperador;
    }

    public function confirmationChangePassword(
        string  $nrOrg,
        string  $email,
        string  $cdOperador,
        string  $dtNascimento,
        string  $dtAdmissao,
        string  $cpf,
        string  $novaSenha,
        string  $novaSenhaConfirmacao
    ): bool {

        $cpfFinal = Util::sanitizeCpf($cpf);
        $defaultIdForConfirmation = 'S';

        // Validacao da confirmacao
        $isConfirmationValid = $this->validaLoginOperador(
            $cdOperador,
            $dtNascimento,
            $dtAdmissao,
            $cpfFinal,
            $defaultIdForConfirmation
        );

        if (empty($isConfirmationValid)) {
            throw new \Exception(Messages::LOGIN_USER_NOT_FOUND);
        }

        if ($novaSenha != $novaSenhaConfirmacao) {
            throw new \Exception(Messages::LOGIN_PASSW_NOT_MATCH);
        }

        // Redefine senha atual
        $this->clearOperatorPassword($nrOrg, $cdOperador);

        // Invoca o modulo de login para atualizaca da senha
        $response = Request::loginRequest(
            'updatePassword',
            [
                'json' => [
                    'requestType' => 'FilterData',
                    'filter' => [
                        [
                            'name' => 'ACTUAL_PASSWORD',
                            'operator' => '=',
                            'value' => self::DEFAULT_PASSWORD,
                        ],
                        [
                            'name' => 'NEW_PASSWORD',
                            'operator' => '=',
                            'value' => $novaSenha,
                        ],
                        [
                            'name' => 'CONFIRM_NEW_PASSWORD',
                            'operator' => '=',
                            'value' => $novaSenhaConfirmacao,
                        ],
                        [
                            'name' => 'EMAIL',
                            'operator' => '=',
                            'value' => $email,
                        ],
                    ]
                ]
            ]
        );
        
        $success = empty($response);
        if (!$success) {
            throw new \Exception(Messages::LOGIN_PASSW_UPDATE_FAIL . ' ' . $response['error']);
        }

        return $success;
    }

    public function validatePassword(
        int $nrOrg,
        string $cdOperador,
        string $password
    ): bool {
        $operador = $this->get($nrOrg, $cdOperador);

        if (!$operador) {
            throw new \Exception('Operador não encontrado');
        }

        $encryptedPassword = md5($password);
        $operadorPassword = $operador['CDSENHAOPERWEB'];

        $isValid = $encryptedPassword === $operadorPassword;

        return $isValid;
    }

    public function getPermissoesModuloOperador($nrorg, $cdoperador) {
        $projeto = 'PF';

        $permissoes = $this->operadorRepository->getPermissoesModuloOperador($nrorg, $cdoperador, $projeto);
    
        // Transformar 'S' em true e 'N' em false
        foreach ($permissoes as &$permissao) {
            $permissao['IDVISUALIZAR'] = $permissao['IDVISUALIZAR'] === 'S';
            $permissao['IDCRIAR'] = $permissao['IDCRIAR'] === 'S';
            $permissao['IDATUALIZAR'] = $permissao['IDATUALIZAR'] === 'S';
            $permissao['IDDELETAR'] = $permissao['IDDELETAR'] === 'S';
        }

        return $permissoes;
    }

    public function controleAcessoAtivo(int $nrOrg) {
        $paramOrg = $this->paramOrgRepository->getParamOrg($nrOrg);
    
        if ($paramOrg && isset($paramOrg['IDATIVACONTROLEACESSO'])) {

            return $paramOrg['IDATIVACONTROLEACESSO'] === "S";
        }
    
        return false;
    }

}